#!/bin/csh
gmt gmtset FONT_LABEL  = 16
gmt gmtset MAP_LABEL_OFFSET     = 0.1i
gmt gmtset FONT_ANNOT_PRIMARY  = 10
gmt gmtset MAP_TICK_PEN_PRIMARY = 1
gmt gmtset FONT_TITLE = 24
gmt gmtset PS_PAGE_ORIENTATION = landscape
gmt gmtset PS_MEDIA A4

set exp=EXP61
set norm_s = 0.2681 # stress normalization factor (P_load from analog experiment)
set norm_l = 0.07883

set out=./${exp}_fig_comparaison.ps

set XZ=./STRESS/${exp}_trajectory_velocity.txt
set vel_filt=./STRESS/${exp}_vfZt.txt
set vel_interp=./STRESS/${exp}_vinterpZt.txt

#~ rm -r ./output_ana
#~ cp -r output ./output_ana

set inpp=./output_defl
set inpp2=./output_nofs
set inpp3=./output_ana

set inpf=./input/${exp}_stress.dat

set Ndis = `awk '/ n_dis output =/ {print $4}' ${inpp}/RUNINFO_01.dat`

#TRAJECTORIES=====================================================================================================================
set Proj=-JX3i/-4.5i
set Range=-R0/1.6/0/1.05
gmt makecpt -Cjet -D -T-0.7/0.7/0.01 > ./palette_stress_Sxx.cpt
awk -v l=$norm_l -v s=$norm_s '{print $1/l, $2/l, -$3/s}' $inpf | gmt surface -G./temp.grd -I0.0025 $Range 
gmt grdimage ./temp.grd $Proj -X0.6i -Y2i -C./palette_stress_Sxx.cpt -K > $out
gmt psbasemap $Proj $Range -Ba0.5f0.25":x*:"/a0.5f0.1g0.5":z*:"WSne -K -O >> $out
awk -v l=$norm_l '{print $2/l, -$3/l}' ${XZ} | gmt psxy $Proj $Range -O -K -W3p,darkbrown,- >> $out
awk  -v ndis=$Ndis -v l=$norm_l '{if (NR>ndis) print $1/l, $2/l}' ${inpp}/XtZtDeE_01.dat | gmt psxy $Proj $Range -O -K -W1p >> $out
awk  -v ndis=$Ndis -v l=$norm_l '{if (NR>ndis) print $1/l, $2/l}' ${inpp2}/XtZtDeE_01.dat | gmt psxy $Proj $Range -O -K -W1p,blue >> $out
awk  -v ndis=$Ndis -v l=$norm_l '{if (NR>ndis) print $1/l, $2/l}' ${inpp3}/XtZtDeE_01.dat | gmt psxy $Proj $Range -O -K -W1p,grey >> $out
gmt psscale -C./palette_stress_Sxx.cpt $Proj $Range -Dg0.0/0.0+w3i/0.1i+h+m -O -K -I0.1 -B0.2":@~s@~@-xx@-*:" -N >> $out
gmt psxy <<FIN -R -J -O -K -S-0.6 -W1p -N >> $out
0.015 1.25
FIN
gmt psxy <<FIN $Proj $Range -S-0.25 -O -K -W3p,darkbrown,- -N >> $out
-0.025 1.30
0.05 1.30
FIN
gmt psxy <<FIN -R -J -O -K -S-0.6 -W1p,blue -N >> $out
0.015 1.35
FIN
gmt psxy <<FIN -R -J -O -K -S-0.6 -W1p,grey -N >> $out
0.015 1.40
FIN
gmt pstext -R -J -O -K -F+f10p,4+jML <<FIN -N >> $out
0.1 1.25 Trajectory (num)
0.1 1.30 Trajectory (analog)
0.1 1.35 Trajectory (num nofs)
0.1 1.40 Trajectory (dyke prop code - analog)
FIN

#ENERGY RELEASE===================================================================================================================
set Range=-R0/0.6/0/1.05
set Proj=-JX1.5i/-4.5i
gmt psbasemap $Range $Proj -X3.3i -Ba0.2f0.1":E (kPa):"/a0.5f0.1g0.5wSne -K -O >> $out
awk  -v ndis=$Ndis -v l=$norm_l '{if (NR>ndis) print $3, $2/l}' $inpp/XtZtDeE_01.dat | gmt psxy -R -J -O -K \
		-Sc0.02 -W1,black >> $out
awk  -v ndis=$Ndis -v l=$norm_l '{if (NR>ndis) print $3, $2/l}' ${inpp2}/XtZtDeE_01.dat | gmt psxy -R -J -O -K \
		-Sc0.02 -W1,blue >> $out
awk  -v ndis=$Ndis -v l=$norm_l '{if (NR>ndis) print $3, $2/l}' ${inpp3}/XtZtDeE_01.dat | gmt psxy -R -J -O -K \
		-Sc0.02 -W1,grey >> $out
gmt psxy -R -J -O -K -Sc0.07 -W1,black <<FIN -N >> $out
0 1.25
FIN
gmt psxy -R -J -O -K -Sc0.07 -W1,grey <<FIN -N >> $out
0 1.35
FIN
gmt pstext -R -J -O -K -F+f10p,4+jML <<FIN -N >> $out
0.15 1.25 Energy release (num)
0.15 1.30 Energy release (num nofs)
0.15 1.35 Energy release (analog)
FIN

#BUOYANCY=========================================================================================================================
set Range=-R0/0.15/0/1.05
gmt psbasemap $Range $Proj -X1.9i -Ba0.05f0.025":@~D@~P (kPa):"/a0.5f0.1g0.5wSne -K -O >> $out
awk  -v l=$norm_l '{print $3, $2/l}' ${inpp}/XtZtDELTA_01.dat | gmt psxy -R -J -O -K -Sc0.03 -W1,black >> $out
awk  -v l=$norm_l '{print $3, $2/l}' ${inpp2}/XtZtDELTA_01.dat | gmt psxy -R -J -O -K -Sc0.03 -W1,blue >> $out
awk  -v l=$norm_l '{print $3, $2/l}' ${inpp3}/XtZtDELTA_01.dat | gmt psxy -R -J -O -K -Sc0.03 -W1,grey >> $out
gmt psxy -R -J -O -K -Sc0.07 -W1,black <<FIN -N >> $out
0 1.25
FIN
gmt psxy -R -J -O -K -Sc0.07 -W1,blue <<FIN -N >> $out
0 1.30
FIN
gmt psxy -R -J -O -K -Sc0.07 -W1,grey <<FIN -N >> $out
0 1.35
FIN
gmt pstext -R -J -O -K -F+f10p,4+jML <<FIN -N >> $out
0.02 1.25 Overpressure (num)
0.02 1.30 Overpressure (num nofs)
0.02 1.35 Overpressure (analog)
FIN

#VELOCITY=========================================================================================================================
set Range=-R0/0.07/0/1.05
gmt psbasemap $Range $Proj -X1.9i -Ba0.025f0.01":v (cm/s):"/a0.5f0.1g0.5wSne -K -O >> $out
awk  -v l=$norm_l '{print $1*1e2, -$2/l}' ${vel_filt} | gmt psxy $Range $Proj -O -K -Sc0.02 -W1,grey >> $out
awk  -v l=$norm_l '{print $1*1e2, -$2/l}' ${vel_interp} | gmt psxy $Range $Proj -O -K -Sc0.01 -W1,red >> $out

#NORMAL STRESS====================================================================================================================
set Range=-R-0.1/0.3/0/1.05
gmt psbasemap $Range $Proj -X1.9i -Ba0.1f0.05":@~s@~@-n@- (kPa):"/a0.5f0.1g0.5wSne -K -O >> $out
awk  -v l=$norm_l '{if(NR%15==5) print $5}' ${inpp}/XtZtDELTAINF_01.txt > ./tmp.Sxx
awk  -v l=$norm_l '{if(NR%15==4) print $5}' ${inpp}/XtZtDELTAINF_01.txt > ./tmp.Zt
paste ./tmp.Sxx ./tmp.Zt > ./tmp.SxxZt
awk  -v l=$norm_l '{print -$4, $2/l}' ${inpp}/XtZtPlitSigNSigT_01.dat | gmt psxy -R -J -O -K -Sc0.03 -W1,black >> $out
awk  -v l=$norm_l '{print -$4, $2/l}' ${inpp2}/XtZtPlitSigNSigT_01.dat | gmt psxy -R -J -O -K -Sc0.03 -W1,blue >> $out
awk  -v l=$norm_l '{print -$4, $2/l}' ${inpp3}/XtZtPlitSigNSigT_01.dat | gmt psxy -R -J -O -K -Sc0.06 -W1,grey >> $out


gmt psxy -R -J -O -K -Sc0.07 -W1,black <<FIN -N >> $out
-0.1 1.25
FIN
gmt psxy -R -J -O -K -Sc0.07 -W1,blue <<FIN -N >> $out
-0.1 1.30
FIN
gmt psxy -R -J -O -K -Sc0.07 -W1,grey <<FIN -N >> $out
-0.1 1.35
FIN


gmt pstext -R -J -O -K -F+f10p,4+jML <<FIN -N >> $out
-0.05 1.25 Normal stress (num-code)
-0.05 1.30 Normal stress (num nofs-code)
-0.05 1.35 Normal stress (analog-code)
FIN


gmt pstext <<fin -R0/11.7/0/8.3 -JX11.7i/8.3i -X-9i -O -F+f14p,4+jML -N -P >> $out
0.1 4.3 a)
3.4 4.3 b)
5.3 4.3 c)
7.2 4.3 d)
9.1 4.3 e)
fin

rm ./tmp.*

gmt psconvert $out -Te -A
