clear all;
close all;
clc;

%Change experiment name 'exp', distance to the center of the load 'dx' and
%the load 'applied load'. Data are in INPUTS_for_EXPERIMENTS/input_experiment_parameters.txt

exp="EXP61";

%Stressfield interpolation needed for the dyke propagaion code. 
% 1) Load the stress field file computed with COMSOL
% 2) Select the value within the desired spatial domain (optionnally within a specified pressure domain)
% 3) Shift the domain to be in the reference frame associated to the load
% 4) Interpolate to a finer grid

pdg=0.0005; % space sampling rate for the grid (m)
dx=0.124; % distance to the center of the load with respect to reference frame centered in the experimental tank (m)
loadapplied=229.6; %applied mass (g)
g=9.81;
surf=0.14*0.06; %surface of the load
L_AN=loadapplied*1e-6*g/surf;  % [kPa] Applied Loading Pressure from Analog Experiment loadapplied*g/surf
L_IN=116.79; % [Pa] Applied Loading Pressure for Comsol simulation

stress=load ('stressfield_rigidload_H04_2D.txt'); % importation des données des valeurs des contraintes

val_R=stress(:,1); % positions r
val_Z=stress(:,2); % positions z
val_sig_x=-stress(:,3); % valeurs sigmas 3
val_sig_z=-stress(:,4);
val_sig_xz=-stress(:,5);

ii=0;
for i=1:length(val_R)
   if ((val_R(i) <= 0.18) && (val_R(i) >= -0.18) && (val_Z(i) <= 0.1) && (val_Z(i) >= -0.19) && (val_sig_x(i) <= L_IN) && (val_sig_x(i) >= -L_IN) && (val_sig_z(i) <= L_IN) && (val_sig_z(i) >= -L_IN) && (val_sig_xz(i) <= L_IN) && (val_sig_xz(i) >= -L_IN) )
       ii=ii+1;
       R(ii,1)=val_R(i)+dx;
       Z(ii,1)=-val_Z(i);
       s_xx(ii,1)=val_sig_x(i)*L_AN; %/L_IN;
       s_zz(ii,1)=val_sig_z(i)*L_AN; %/L_IN;
       s_xz(ii,1)=-val_sig_xz(i)*L_AN; %/L_IN;
   end
end

%% Mise en place grille interpolation
xmin=0.0;   % taille de la grille d'interpolation en m
xmax=0.18; 
zmax=0.18;
zmin=0.0;

Rbis=xmin:pdg:xmax; % limitation de la zone d'interpolation en r (correspond à de 0 a 14.5 cm)
Zbis=zmin:pdg:zmax; % limitation de la zone d'interpolation en z (correspond à un hauteur max de 6 cm et une profondeur max de 10 cm)
[Rr,Zz]=meshgrid(Rbis,Zbis); % mise en place de la grille d'interpolation
inter_sigx=griddata(R,Z,s_xx,Rr,Zz,'cubic'); % interpolation sigma 3
inter_sigz=griddata(R,Z,s_zz,Rr,Zz,'cubic'); % interpolation sigma 3
inter_sigxz=griddata(R,Z,s_xz,Rr,Zz,'cubic'); % interpolation sigma 3

fileID1=fopen(['' num2str(exp) '_stress.dat'],'w');
% fileID2=fopen(['' num2str(exp) '_stress_zero_stress.dat'],'w');
for j=length(Rr(:,1)):-1:1
   for i=1:length(Rr(1,:))
       fprintf(fileID1, '%12.6f %12.6f %12.6f %12.6f %12.6f \n', Rr(j,i), Zz(j,i), -inter_sigx(j,i),-inter_sigz(j,i),-inter_sigxz(j,i));
%        fprintf(fileID2, '%12.6f %12.6f %12.6f %12.6f %12.6f \n', Rr(j,i), Zz(j,i), 0,0,0);
   end
end
fclose(fileID1);
% fclose(fileID2);


figure()
hold on
a=pcolor(Rr,-Zz,inter_sigxz/L_AN);
caxis([-0.40 0.40])
  colormap(jet)
  color=colorbar ;
  shading interp
  alpha(a,0.5);
%  v=[0.02:0.02:0.6];
